@@ -341,6 +341,7 @@ sub conddup {
     my ($self, $code) = @_;
     my $rv = eval { $code->(); };
     throw("dup") if $self->was_duplicate_error;
+    croak($@) if $@;
     return $rv;
 }
 
@@ -735,6 +736,8 @@ sub delete_host {
 # return true if deleted, 0 if didn't exist, exception if error
 sub delete_domain {
     my ($self, $dmid) = @_;
+    throw("has_files")   if $self->domain_has_files($dmid);
+    throw("has_classes") if $self->domain_has_classes($dmid);
     return $self->dbh->do("DELETE FROM domain WHERE dmid = ?", undef, $dmid);
 }
 
@@ -745,6 +748,13 @@ sub domain_has_files {
     return $has_a_fid ? 1 : 0;
 }
 
+sub domain_has_classes {
+    my ($self, $dmid) = @_;
+    my $has_a_class = $self->dbh->selectrow_array('SELECT classid FROM class WHERE dmid = ? LIMIT 1',
+        undef, $dmid);
+    return $has_a_class ? 1 : 0;
+}
+
 sub class_has_files {
     my ($self, $dmid, $clid) = @_;
     my $has_a_fid = $self->dbh->selectrow_array('SELECT fid FROM file WHERE dmid = ? AND classid = ? LIMIT 1',
@@ -1042,6 +1052,18 @@ sub create_device {
     return 1;
 }
 
+sub update_device {
+    my ($self, $devid, $to_update) = @_;
+    my @keys = sort keys %$to_update;
+    return unless @keys;
+    $self->conddup(sub {
+        $self->dbh->do("UPDATE device SET " . join('=?, ', @keys)
+            . "=? WHERE devid=?", undef, (map { $to_update->{$_} } @keys),
+            $devid);
+    });
+    return 1;
+}
+
 sub update_device_usage {
     my $self = shift;
     my %arg  = $self->_valid_params([qw(mb_total mb_used devid)], @_);
@@ -1052,6 +1074,19 @@ sub update_device_usage {
     $self->condthrow;
 }
 
+# This is unimplemented at the moment as we must verify:
+# - no file_on rows exist
+# - nothing in file_to_queue is going to attempt to use it
+# - nothing in file_to_replicate is going to attempt to use it
+# - it's already been marked dead
+# - that all trackers are likely to know this :/
+# - ensure the devid can't be reused
+# IE; the user can't mark it dead then remove it all at once and cause their
+# cluster to implode.
+sub delete_device {
+    die "Unimplemented; needs further testing";
+}
+
 sub mark_fidid_unreachable {
     my ($self, $fidid) = @_;
     die "Your database does not support REPLACE! Reimplement mark_fidid_unreachable!" unless $self->can_replace;
@@ -1077,6 +1112,7 @@ sub set_device_state {
 
 sub delete_class {
     my ($self, $dmid, $cid) = @_;
+    throw("has_files") if $self->class_has_files($dmid, $cid);
     eval {
         $self->dbh->do("DELETE FROM class WHERE dmid = ? AND classid = ?", undef, $dmid, $cid);
     };
@@ -1143,6 +1179,13 @@ sub rename_file {
     return 1;
 }
 
+sub get_domainid_by_name {
+    my $self = shift;
+    my ($dmid) = $self->dbh->selectrow_array('SELECT dmid FROM domain WHERE namespace = ?',
+        undef, $_[0]);
+    return $dmid;
+}
+
 # returns a hash of domains. Key is namespace, value is dmid.
 sub get_all_domains {
     my ($self) = @_;
@@ -1150,6 +1193,13 @@ sub get_all_domains {
     return map { ($_->[0], $_->[1]) } @{$domains || []};
 }
 
+sub get_classid_by_name {
+    my $self = shift;
+    my ($classid) = $self->dbh->selectrow_array('SELECT classid FROM class WHERE dmid = ? AND classname = ?',
+        undef, $_[0], $_[1]);
+    return $classid;
+}
+
 # returns an array of hashrefs, one hashref per row in the 'class' table
 sub get_all_classes {
     my ($self) = @_;
@@ -1192,6 +1242,21 @@ sub remove_fidid_from_devid {
     return $rv;
 }
 
+# Test if host exists.
+sub get_hostid_by_id {
+    my $self = shift;
+    my ($hostid) = $self->dbh->selectrow_array('SELECT hostid FROM host WHERE hostid = ?',
+        undef, $_[0]);
+    return $hostid;
+}
+
+sub get_hostid_by_name {
+    my $self = shift;
+    my ($hostid) = $self->dbh->selectrow_array('SELECT hostid FROM host WHERE hostname = ?',
+        undef, $_[0]);
+    return $hostid;
+}
+
 # get all hosts from database, returns them as list of hashrefs, hashrefs being the row contents.
 sub get_all_hosts {
     my ($self) = @_;
@@ -1441,6 +1506,18 @@ sub create_domain {
     die "failed to make domain";  # FIXME: the above is racy.
 }
 
+sub update_host {
+    my ($self, $hid, $to_update) = @_;
+    my @keys = sort keys %$to_update;
+    return unless @keys;
+    $self->conddup(sub {
+        $self->dbh->do("UPDATE host SET " . join('=?, ', @keys)
+            . "=? WHERE hostid=?", undef, (map { $to_update->{$_} } @keys),
+            $hid);
+    });
+    return 1;
+}
+
 sub update_host_property {
     my ($self, $hostid, $col, $val) = @_;
     $self->conddup(sub {
