@@ -164,7 +164,7 @@ sub check_domain {
     return $self->err_line("no_domain") unless defined $domain && length $domain;
 
     # validate domain
-    my $dmid = MogileFS::Domain->id_of_name($domain) or
+    my $dmid = eval { Mgd::domain_factory()->get_by_name($domain)->id } or
         return $self->err_line("unreg_domain");
 
     return $dmid;
@@ -188,10 +188,7 @@ sub cmd_clear_cache {
     my MogileFS::Worker::Query $self = shift;
     my $args = shift;
 
-    MogileFS::Device->invalidate_cache if $args->{devices} || $args->{all};
-    MogileFS::Host->invalidate_cache   if $args->{hosts}   || $args->{all};
-    MogileFS::Class->invalidate_cache  if $args->{class}   || $args->{all};
-    MogileFS::Domain->invalidate_cache if $args->{domain}  || $args->{all};
+    # TODO: Use this to tell Monitor worker to rebroadcast all state
 
     return $self->ok_line;
 }
@@ -234,7 +231,7 @@ sub cmd_create_open {
     my $class = $args->{class} || "";
     my $classid = 0;
     if (length($class)) {
-        $classid = MogileFS::Class->class_id($dmid, $class)
+        $classid = eval { Mgd::class_factory()->get_by_name($dmid, $class)->id }
             or return $self->err_line("unreg_class");
     }
 
@@ -248,8 +245,8 @@ sub cmd_create_open {
 
     my @devices;
 
-    unless (MogileFS::run_global_hook('cmd_create_open_order_devices', [MogileFS::Device->devices], \@devices)) {
-        @devices = sort_devs_by_freespace(MogileFS::Device->devices);
+    unless (MogileFS::run_global_hook('cmd_create_open_order_devices', [Mgd::device_factory()->get_all], \@devices)) {
+        @devices = sort_devs_by_freespace(Mgd::device_factory()->get_all);
     }
 
     # find suitable device(s) to put this file on.
@@ -330,7 +327,6 @@ sub sort_devs_by_freespace {
     } sort {
         $b->percent_free <=> $a->percent_free;
     } grep {
-        $_->exists &&
         $_->should_get_new_files;
     } @_;
 
@@ -464,7 +460,7 @@ sub cmd_updateclass {
     my $key   = $args->{key}        or return $self->err_line("no_key");
     my $class = $args->{class}      or return $self->err_line("no_class");
 
-    my $classid = MogileFS::Class->class_id($dmid, $class)
+    my $classid = eval { Mgd::class_factory()->get_by_name($dmid, $class)->id }
         or return $self->err_line('class_not_found');
 
     my $fid = MogileFS::FID->new_from_dmid_and_key($dmid, $key)
@@ -533,9 +529,9 @@ sub cmd_file_debug {
     }
 
     if ($fid) {
-        $fid->{domain}   = MogileFS::Domain->name_of_id($fid->{dmid});
-        $fid->{class}    = MogileFS::Class->class_name($fid->{dmid},
-            $fid->{classid});
+        $fid->{domain}   = Mgd::domain_factory()->get_by_id($fid->{dmid})->name;
+        $fid->{class}    = Mgd::class_factory()->get_by_id($fid->{dmid},
+            $fid->{classid})->name;
     }
 
     # Fetch all of the queue data.
@@ -589,8 +585,9 @@ sub cmd_file_info {
 
     my $ret = {};
     $ret->{fid}      = $fid->id;
-    $ret->{domain}   = MogileFS::Domain->name_of_id($fid->dmid);
-    $ret->{class}    = MogileFS::Class->class_name($fid->dmid, $fid->classid);
+    $ret->{domain}   = Mgd::domain_factory()->get_by_id($fid->dmid)->name;
+    $ret->{class}    = Mgd::class_factory()->get_by_id($fid->dmid,
+        $fid->classid)->name;
     $ret->{key}      = $key;
     $ret->{'length'} = $fid->length;
     $ret->{devcount} = $fid->devcount;
@@ -627,8 +624,10 @@ sub cmd_list_fids {
         $ct++;
         my $fid = $r->{fid};
         $ret->{"fid_${ct}_fid"} = $fid;
-        $ret->{"fid_${ct}_domain"} = ($domains{$r->{dmid}} ||= MogileFS::Domain->name_of_id($r->{dmid}));
-        $ret->{"fid_${ct}_class"} = ($classes{$r->{dmid}}{$r->{classid}} ||= MogileFS::Class->class_name($r->{dmid}, $r->{classid}));
+        $ret->{"fid_${ct}_domain"} = ($domains{$r->{dmid}} ||=
+            Mgd::domain_factory->get_by_id($r->{dmid})->name);
+        $ret->{"fid_${ct}_class"} = ($classes{$r->{dmid}}{$r->{classid}} ||=
+            Mgd::class_factory->get_by_id($r->{dmid}, $r->{classid})->name);
         $ret->{"fid_${ct}_key"} = $r->{dkey};
         $ret->{"fid_${ct}_length"} = $r->{length};
         $ret->{"fid_${ct}_devcount"} = $r->{devcount};
@@ -702,18 +701,15 @@ sub cmd_get_hosts {
     my MogileFS::Worker::Query $self = shift;
     my $args = shift;
 
-    MogileFS::Host->invalidate_cache;
-
     my $ret = { hosts => 0 };
-    foreach my $host (MogileFS::Host->hosts) {
+    for my $host (Mgd::host_factory()->get_all) {
         next if defined $args->{hostid} && $host->id != $args->{hostid};
         my $n = ++$ret->{hosts};
-        foreach my $key (qw(hostid status hostname hostip
-                            http_port http_get_port
-                            altip altmask))
-        {
+        my $fields = $host->fields(qw(hostid status hostname hostip http_port
+            http_get_port altip altmask));
+        while (my ($key, $val) = each %$fields) {
             # must be regular data so copy it in
-            $ret->{"host${n}_$key"} = $host->field($key);
+            $ret->{"host${n}_$key"} = $val;
         }
     }
 
@@ -725,11 +721,11 @@ sub cmd_get_devices {
     my $args = shift;
 
     my $ret = { devices => 0 };
-    foreach my $dev (MogileFS::Device->devices) {
+    for my $dev (Mgd::device_factory()->get_all) {
         next if defined $args->{devid} && $dev->id != $args->{devid};
         my $n = ++$ret->{devices};
 
-        my $sum = $dev->overview_hashref;
+        my $sum = $dev->fields;
         while (my ($key, $val) = each %$sum) {
             $ret->{"dev${n}_$key"} = $val;
         }
@@ -749,24 +745,20 @@ sub cmd_create_device {
     my $devid = $args->{devid};
     return $self->err_line("invalid_devid") unless $devid && $devid =~ /^\d+$/;
 
-    my ($host, $hostid);
+    my $hostid;
 
-    MogileFS::Host->check_cache;
+    my $sto = Mgd::get_store();
     if ($args->{hostid} && $args->{hostid} =~ /^\d+$/) {
-        $hostid = $args->{hostid};
-        $host = MogileFS::Host->of_hostid($hostid);
-        return $self->err_line("unknown_hostid") unless $host && $host->exists;
+        $hostid = $sto->get_hostid_by_id($args->{hostid});
+        return $self->err_line("unknown_hostid") unless $hostid;
     } elsif (my $hname = $args->{hostname}) {
-        $host = MogileFS::Host->of_hostname($hname);
-        return $self->err_line("unknown_host") unless $host;
-        $hostid = $host->id;
+        $hostid = $sto->get_hostid_by_name($hname);
+        return $self->err_line("unknown_host") unless $hostid;
     } else {
         return $self->err_line("bad_args", "No hostid/hostname parameter");
     }
 
-    if (eval { MogileFS::Device->create(devid  => $devid,
-                                        hostid => $hostid,
-                                        status => $status) }) {
+    if (eval { $sto->create_device($devid, $hostid, $status) }) {
         return $self->ok_line;
     }
 
@@ -782,9 +774,7 @@ sub cmd_create_domain {
     my $domain = $args->{domain} or
         return $self->err_line('no_domain');
 
-    # TODO: auth/permissions?
-
-    my $dom = eval { MogileFS::Domain->create($domain); };
+    my $dom = eval { Mgd::get_store()->create_domain($domain); };
     if ($@) {
         if (error_code($@) eq "dup") {
             return $self->err_line('domain_exists');
@@ -802,10 +792,11 @@ sub cmd_delete_domain {
     my $domain = $args->{domain} or
         return $self->err_line('no_domain');
 
-    my $dom = MogileFS::Domain->of_namespace($domain) or
+    my $sto = Mgd::get_store();
+    my $dmid = $sto->get_domainid_by_name($domain) or
         return $self->err_line('domain_not_found');
 
-    if (eval { $dom->delete }) {
+    if (eval { $sto->delete_domain($dmid) }) {
         return $self->ok_line({ domain => $domain });
     }
 
@@ -835,20 +826,29 @@ sub cmd_create_class {
         return $self->err_line('invalid_replpolicy', $@) if $@;
     }
 
-    my $dom  = MogileFS::Domain->of_namespace($domain) or
+    my $sto = Mgd::get_store();
+    my $dmid  = $sto->get_domainid_by_name($domain) or
         return $self->err_line('domain_not_found');
 
-    my $cls = $dom->class($class);
+    my $clsid = $sto->get_classid_by_name($dmid, $class);
     if ($args->{update}) {
-        return $self->err_line('class_not_found') if ! $cls;
-        $cls->set_name($class);
+        return $self->err_line('class_not_found') if ! $clsid;
+        $sto->update_class_name(dmid => $dmid, classid => $clsid,
+            classname => $class);
     } else {
-        return $self->err_line('class_exists') if $cls;
-        $cls = $dom->create_class($class);
+        $clsid = eval { $sto->create_class($dmid, $class); };
+        if ($@) {
+            if (error_code($@) eq "dup") {
+                return $self->err_line('class_exists');
+            }
+            return $self->err_line('failure', "$@");
+        }
     }
-    $cls->set_mindevcount($mindevcount);
+    $sto->update_class_mindevcount(dmid => $dmid, classid => $clsid,
+        mindevcount => $mindevcount);
     # don't erase an existing replpolicy if we're not setting a new one.
-    $cls->set_replpolicy($replpolicy) if $replpolicy;
+    $sto->update_class_replpolicy(dmid => $dmid, classid => $clsid,
+        replpolicy => $replpolicy) if $replpolicy;
 
     # return success
     return $self->ok_line({ class => $class, mindevcount => $mindevcount, domain => $domain });
@@ -871,12 +871,13 @@ sub cmd_delete_class {
     my $class = $args->{class};
     return $self->err_line('no_class') unless length $domain;
 
-    my $dom  = MogileFS::Domain->of_namespace($domain) or
+    my $sto = Mgd::get_store();
+    my $dmid  = $sto->get_domainid_by_name($domain) or
         return $self->err_line('domain_not_found');
-    my $cls = $dom->class($class) or
+    my $clsid = $sto->get_classid_by_name($dmid, $class) or
         return $self->err_line('class_not_found');
 
-    if (eval { $cls->delete }) {
+    if (eval { Mgd::get_store()->delete_class($dmid, $clsid) }) {
         return $self->ok_line({ domain => $domain, class => $class });
     }
 
@@ -892,14 +893,15 @@ sub cmd_create_host {
     my $hostname = $args->{host} or
         return $self->err_line('no_host');
 
-    my $host = MogileFS::Host->of_hostname($hostname);
+    my $sto = Mgd::get_store();
+    my $hostid = $sto->get_hostid_by_name($hostname);
 
     # if we're creating a new host, require ip/port, and default to
     # host being down if client didn't specify
     if ($args->{update}) {
-        return $self->err_line('host_not_found') unless $host;
+        return $self->err_line('host_not_found') unless $hostid;
     } else {
-        return $self->err_line('host_exists') if $host;
+        return $self->err_line('host_exists') if $hostid;
         return $self->err_line('no_ip') unless $args->{ip};
         return $self->err_line('no_port') unless $args->{port};
         $args->{status} ||= 'down';
@@ -907,26 +909,23 @@ sub cmd_create_host {
 
     if ($args->{status}) {
         return $self->err_line('unknown_state')
-            unless MogileFS::Host->valid_initial_state($args->{status});
+            unless MogileFS::Host->valid_state($args->{status});
     }
 
     # arguments all good, let's do it.
 
-    $host ||= MogileFS::Host->create($hostname, $args->{ip});
-    my %setter = (
-                  status  => "set_status",
-                  ip      => "set_ip",
-                  port    => "set_http_port",
-                  getport => "set_http_get_port",
-                  altip   => "set_alt_ip",
-                  altmask => "set_alt_mask",
-                  );
-    while (my ($f, $meth) = each %setter) {
-        $host->$meth($args->{$f}) if exists $args->{$f};
-    }
+    $hostid ||= $sto->create_host($hostname, $args->{ip});
+
+    # Protocol mismatch data fixup.
+    $args->{hostip} = delete $args->{ip} if exists $args->{ip};
+    $args->{http_port} = delete $args->{port} if exists $args->{port};
+    $args->{http_get_port} = delete $args->{getport} if exists $args->{getport};
+    my @toupdate = grep { exists $args->{$_} } qw(status hostip http_port
+        http_get_port altip altmask);
+    $sto->update_host($hostid, { map { $_ => $args->{$_} } @toupdate });
 
     # return success
-    return $self->ok_line($host->overview_hashref);
+    return $self->ok_line({ hostid => $hostid, hostname => $hostname });
 }
 
 sub cmd_update_host {
@@ -941,17 +940,17 @@ sub cmd_delete_host {
     my MogileFS::Worker::Query $self = shift;
     my $args = shift;
 
-    my $host   = MogileFS::Host->of_hostname($args->{host})
+    my $sto = Mgd::get_store();
+    my $hostid = $sto->get_hostid_by_name($args->{host})
         or return $self->err_line('unknown_host');
 
-    my $hostid = $host->id;
-
-    foreach my $dev (MogileFS::Device->devices) {
+    # TODO: $sto->delete_host should have a "has_devices" test internally
+    for my $dev ($sto->get_all_devices) {
         return $self->err_line('host_not_empty')
-            if $dev->hostid == $hostid;
+            if $dev->{hostid} == $hostid;
     }
 
-    $host->delete;
+    $sto->delete_host($hostid);
 
     return $self->ok_line;
 }
@@ -960,11 +959,9 @@ sub cmd_get_domains {
     my MogileFS::Worker::Query $self = shift;
     my $args = shift;
 
-    MogileFS::Domain->invalidate_cache;
-
     my $ret = {};
     my $dm_n = 0;
-    foreach my $dom (MogileFS::Domain->domains) {
+    for my $dom (Mgd::domain_factory()->get_all) {
         $dm_n++;
         $ret->{"domain${dm_n}"} = $dom->name;
         my $cl_n = 0;
@@ -1034,7 +1031,7 @@ sub cmd_get_paths {
     # add to memcache, if needed.  for an hour.
     $memc->add($mogfid_memkey, $fid->id, 3600) if $need_fid_in_memcache;
 
-    my $dmap = MogileFS::Device->map;
+    my $dmap = Mgd::device_factory()->map_by_id;
 
     my $ret = {
         paths => 0,
@@ -1189,7 +1186,7 @@ sub cmd_edit_file {
     # add to memcache, if needed.  for an hour.
     $memc->add($mogfid_memkey, $fid->id, 3600) if $need_fid_in_memcache;
 
-    my $dmap = MogileFS::Device->map;
+    my $dmap = Mgd::device_factory()->map_by_id;
 
     my @devices_with_weights;
 
@@ -1245,13 +1242,13 @@ sub cmd_edit_file {
     # Take first remaining device from list
     my $devid = $list[0];
 
-    my $class = MogileFS::Class->of_fid($fid);
+    my $classid = $fid->classid;
     my $newfid = eval {
         Mgd::get_store()->register_tempfile(
             fid     => undef,   # undef => let the store pick a fid
             dmid    => $dmid,
             key     => $key,    # This tempfile will ultimately become this key
-            classid => $class->classid,
+            classid => $classid,
             devids  => $devid,
         );
     };
@@ -1279,7 +1276,7 @@ sub cmd_edit_file {
     $ret->{newpath} = $paths[1];
     $ret->{fid} = $newfid;
     $ret->{devid} = $devid;
-    $ret->{class} = $class->classid;
+    $ret->{class} = $classid;
     return $self->ok_line($ret);
 }
 
@@ -1292,8 +1289,10 @@ sub cmd_set_weight {
     return $self->err_line('bad_params')
         unless $hostname && $devid && $weight >= 0;
 
-    my $dev = MogileFS::Device->from_devid_and_hostname($devid, $hostname)
-        or return $self->err_line('host_mismatch');
+    my $dev = Mgd::device_factory()->get_by_id($devid);
+    return $self->err_line('no_device') unless $dev;
+    return $self->err_line('host_mismatch')
+        unless $dev->host->hostname eq $hostname;
 
     $dev->set_weight($weight);
 
@@ -1311,14 +1310,16 @@ sub cmd_set_state {
     return $self->err_line('bad_params')
         unless $hostname && $devid && $dstate;
 
-    my $dev = MogileFS::Device->from_devid_and_hostname($devid, $hostname)
-        or return $self->err_line('host_mismatch');
+    my $dev = Mgd::device_factory()->get_by_id($devid);
+    return $self->err_line('no_device') unless $dev;
+    return $self->err_line('host_mismatch')
+        unless $dev->host->hostname eq $hostname;
 
     # make sure the destination state isn't too high
     return $self->err_line('state_too_high')
         unless $dev->can_change_to_state($state);
 
-    $dev->set_state($state);
+    Mgd::get_store()->set_device_state($dev->id, $state);
     return $self->ok_line;
 }
 
@@ -1563,7 +1564,7 @@ sub cmd_rebalance_start {
 
         my $rebal = MogileFS::Rebalance->new;
         $rebal->policy($rebal_pol);
-        my @devs  = MogileFS::Device->devices;
+        my @devs  = Mgd::device_factory()->get_all;
         $rebal->init(\@devs);
         my $sdevs = $rebal->source_devices;
 
@@ -1582,7 +1583,7 @@ sub cmd_rebalance_test {
     return $self->err_line('no_rebal_policy') unless $rebal_pol;
 
     my $rebal = MogileFS::Rebalance->new;
-    my @devs  = MogileFS::Device->devices;
+    my @devs  = Mgd::device_factory()->get_all;
     $rebal->policy($rebal_pol);
     $rebal->init(\@devs);
 
@@ -1685,6 +1686,7 @@ sub err_line {
         'key_exists' => "Target key name already exists; can't overwrite.",
         'no_class' => "No class provided",
         'no_devices' => "No devices found to store file",
+        'no_device' => "Device not found",
         'no_domain' => "No domain provided",
         'no_host' => "No host provided",
         'no_ip' => "IP required to create host",
